@@ -193,15 +193,15 @@ enum ContainerImageOpts {
         proxyopts: ContainerProxyOpts,
     },
 
-    /// Pull (or update) a container image.
+    /// Output metadata about an already stored container image.
     History {
         /// Path to the repository
         #[structopt(long, parse(try_from_str = parse_repo))]
         repo: ostree::Repo,
 
-        /// Image reference, e.g. ostree-remote-image:someremote:registry:quay.io/exampleos/exampleos:latest
-        #[structopt(parse(try_from_str = parse_imgref))]
-        imgref: OstreeImageReference,
+        /// Container image reference, e.g. registry:quay.io/exampleos/exampleos:latest
+        #[structopt(parse(try_from_str = parse_base_imgref))]
+        imgref: ImageReference,
     },
 
     /// Copy a pulled container image from one repo to another.
@@ -221,6 +221,22 @@ enum ContainerImageOpts {
         imgref: OstreeImageReference,
     },
 
+    /// Unreference one or more pulled container images and perform a garbage collection.
+    Remove {
+        /// Path to the repository
+        #[structopt(long)]
+        #[structopt(parse(try_from_str = parse_repo))]
+        repo: ostree::Repo,
+
+        /// Image reference, e.g. quay.io/exampleos/exampleos:latest
+        #[structopt(parse(try_from_str = parse_base_imgref))]
+        imgrefs: Vec<ImageReference>,
+
+        /// Do not garbage collect unused layers
+        #[structopt(long)]
+        skip_gc: bool,
+    },
+
     /// Perform initial deployment for a container image
     Deploy {
         /// Path to the system root
@@ -286,6 +302,8 @@ enum TestingOpts {
     DetectEnv,
     /// Execute integration tests, assuming mutable environment
     Run,
+    /// Execute IMA tests
+    RunIMA,
     FilterTar,
 }
 
@@ -542,8 +560,8 @@ fn print_column(s: &str, clen: usize, remaining: &mut usize) {
 }
 
 /// Output the container image history
-async fn container_history(repo: &ostree::Repo, imgref: &OstreeImageReference) -> Result<()> {
-    let img = crate::container::store::query_image(repo, imgref)?
+async fn container_history(repo: &ostree::Repo, imgref: &ImageReference) -> Result<()> {
+    let img = crate::container::store::query_image_ref(repo, imgref)?
         .ok_or_else(|| anyhow::anyhow!("No such image: {}", imgref))?;
     let columns = [("ID", 20), ("SIZE", 10), ("CREATED BY", 0usize)];
     let width = term_size::dimensions().map(|x| x.0).unwrap_or(80);
@@ -612,6 +630,7 @@ fn testing(opts: &TestingOpts) -> Result<()> {
             Ok(())
         }
         TestingOpts::Run => crate::integrationtest::run_tests(),
+        TestingOpts::RunIMA => crate::integrationtest::test_ima(),
         TestingOpts::FilterTar => {
             crate::tar::filter_tar(std::io::stdin(), std::io::stdout()).map(|_| {})
         }
@@ -672,6 +691,21 @@ where
                 ContainerImageOpts::History { repo, imgref } => {
                     container_history(&repo, &imgref).await
                 }
+                ContainerImageOpts::Remove {
+                    repo,
+                    imgrefs,
+                    skip_gc,
+                } => {
+                    let nimgs = imgrefs.len();
+                    crate::container::store::remove_images(&repo, imgrefs.iter())?;
+                    if !skip_gc {
+                        let nlayers = crate::container::store::gc_image_layers(&repo)?;
+                        println!("Removed images: {nimgs} layers: {nlayers}");
+                    } else {
+                        println!("Removed images: {nimgs}");
+                    }
+                    Ok(())
+                }
                 ContainerImageOpts::Copy {
                     src_repo,
                     dest_repo,
